//------------------------------------------------------------------------------
//
//  npmapfile.c
//
//  ANTz application based on NPE
//
//  Created by Shane Saxon on 3/13/11.
//  Copyright Saxon Digital 2010-2011. All rights reserved.
//
//-----------------------------------------------------------------------------

#include "npmapfile.h"

#include <stdlib.h>

#include "../os/npos.h"
#include "../io/npfile.h"
#include "../npdata.h"

#include "nptypes.h"

//-----------------------------------------------------------------------------
void npInitMapFile (void* dataRef)
{

}

//-----------------------------------------------------------------------------
void npCloseMapFile (void* dataRef)
{

}

//-----------------------------------------------------------------------------
// mode usage: 3 parameters "r/w/a" followed by optional "b" and optional "+"
// example mode = "rb+"
// r open file for reading
// w open file for overwriting. Create new file if doesn't already exist
// a open file for appending data. 
// b open file as a binary file
// + open file for read and writing
//-----------------------------------------------------------------------------
FILE* npOpenMapFile (const char* fileName, const char* mode, void* dataRef)
{
	FILE* file = NULL;

	file = fopen(fileName, mode); 

	if ( file == NULL )
	{
		perror("Failed to open/create file");
		return NULL;
	}

	// add code to add the file ptr to the data->map							debug, zz

	return file;
}

//-----------------------------------------------------------------------------
FILE* npNewMapFile (const char* fileName, const char* mode, void* dataRef)
{
}


//-----------------------------------------------------------------------------
int npReadMapFile (void* readBuffer, int elementSize, int elementCount, 
				 FILE* file, void* dataRef)
{
	int bytes = 0;

	bytes = fread (readBuffer, elementSize, elementCount, file);
	
	if ( bytes == 0 )
		printf("Failed Reading\n");
	else
		printf("Success Reading\n");

	return bytes;
}


//-----------------------------------------------------------------------------
FILE* npDialogMapFile (const char* fileName, int dialogType, void* dataRef)
{	
	
	switch (dialogType)
	{
		case kNPfileDialogNew : ; break;

		case kNPfileDialogOpen : ; break;
		case kNPfileDialogClose : ; break;

		case kNPfileDialogSave : ; break;
		case kNPfileDialogSaveAs : ; break;

		case kNPfileDialogImport : ; break;
		case kNPfileDialogExport : ; break;

		default : break;
	}

	return NULL;
}


//-----------------------------------------------------------------------------
int npWriteMapFile (const void* str, int wordSize, int size, FILE* file, void* dataRef)
{
	return fwrite( str, wordSize, strlen(str), file );
}



void npSaveMapFile (FILE* file, void* dataRef)
{

}


//-----------------------------------------------------------------------------
int npImportMapFile (FILE* file, int type, void* dataRef)
{
	int size = 0;
	return size;
}

//------------------------------------------------------------------------------
int npMapTraverseTree (char* writeBuffer, NPplotPtr node, void* dataRef)
{
	int i = 0;
	int count = 0;

	// if not root then write the data, root has already been written
	if (node->branchLevel > 0)
			count += npWriteNode (writeBuffer, node, dataRef);


	//recursively calls this function for all children
	for (i=0; i < node->childCount; i++)//node->childCount; j++)
	{
		if (node->child[i] != NULL)
			count += npMapTraverseTree ((writeBuffer+count), node->child[i], dataRef);	
	}

	return count;
}

//-----------------------------------------------------------------------------
// write the header structure
int npWriteMapHeader (char* bufferIndex, int count, int type, void* dataRef)
{
	int i = 0;
	int itemCount = 0;

	NPmapTypePtr mapPtr= NULL;

	pData data = (pData) dataRef;

	count = 0;


	// need first one without comma
	count += sprintf ((bufferIndex + count), "%s", data->map.typeMapPlot[0].name);

	switch (type)
	{
		case kNPplot: itemCount = 46; break;
		case kNPcamera: itemCount = 18; break;
		case kNPgrid: itemCount = 6; break;
		case kNPpin: itemCount = 13; break;

		default : break;
	}

	for (i=1; i < itemCount ;i++)
	{
		// insert a comma for CSV and JSON files
		count += sprintf ((bufferIndex + count), ",");

		switch (type)
		{
			case kNPplot:
		

		switch (data->map.typeMapPlot[i].type)
		{
			case 0 :				// same as default
				count += sprintf ((bufferIndex + count), "%s", 
									data->map.typeMapPlot[i].name );
				break;
			case kNPfloatXYZ : 
				count += sprintf ((bufferIndex + count), "%s x,%s y,%s z", 
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name );
				break;
			case kNPintXYZ : 
				count += sprintf ((bufferIndex + count), "%s x,%s y,%s z", 
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name );
				break;
			case kNPboolXYZ : 
				count += sprintf ((bufferIndex + count), "%s x,%s y,%s z", 
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name );	
				break;
			case kNPfloatXYZA : 
				count += sprintf ((bufferIndex + count), "%s x,%s y,%s z,%s a", 
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name );
			case kNPfloatXYZS : 
				count += sprintf ((bufferIndex + count), "%s x,%s y,%s z,%s s", 
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name );	
				break;
			case kNPboolXYZS : 
				count += sprintf ((bufferIndex + count), "%s x,%s y,%s z,%s s", 
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name );	
				break;
			case kNPubyteRGBA : 
				count += sprintf ((bufferIndex + count), "%s r,%s g,%s b,%s a", 
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name );	
				break;
			case kNPubyteRGB : 
				count += sprintf ((bufferIndex + count), "%s r,%s g,%s b", 
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name );	
				break;
			case kNPfloatRGBA : 
				count += sprintf ((bufferIndex + count), "%s r,%s g,%s b,%s a", 
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name );	
				break;
			case kNPfloatRGB : 
				count += sprintf ((bufferIndex + count), "%s r,%s g,%s b", 
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name,
									data->map.typeMapPlot[i].name );	
				break;

			default :				// same as case 0
				count += sprintf ((bufferIndex + count), "%s", 
									data->map.typeMapPlot[i].name );
				break;
		}
		break;
			
		case kNPcamera:
			count += sprintf ((bufferIndex + count), "%s",
				data->map.typeMapCamera[i].name );
			break;
		case kNPgrid :
			switch (data->map.typeMapGrid[i].type)
			{
				case kNPfloatXYZ : 
				case kNPintXYZ : 
					count += sprintf ((bufferIndex + count), "%s x,%s y,%s z", 
									data->map.typeMapGrid[i].name,
									data->map.typeMapGrid[i].name,
									data->map.typeMapGrid[i].name );
					break;
				default :				// same as case 0
					count += sprintf ((bufferIndex + count), "%s", 
									data->map.typeMapGrid[i].name );
				break;
			}
			break;
		case kNPpin: 
			switch (data->map.typeMapPin[i].type)
			{
				case 0 :				// same as default
					count += sprintf ((bufferIndex + count), "%s", 
									data->map.typeMapPin[i].name );
					break;
				case kNPfloatXYZ : 
					count += sprintf ((bufferIndex + count), "%s x,%s y,%s z", 
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name );
					break;
				case kNPintXYZ : 
				count += sprintf ((bufferIndex + count), "%s x,%s y,%s z", 
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name );
				break;
			case kNPboolXYZ : 
				count += sprintf ((bufferIndex + count), "%s x,%s y,%s z", 
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name );	
				break;
			case kNPfloatXYZA : 
				count += sprintf ((bufferIndex + count), "%s x,%s y,%s z,%s a", 
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name );
			case kNPfloatXYZS : 
				count += sprintf ((bufferIndex + count), "%s x,%s y,%s z,%s s", 
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name );	
				break;
			case kNPboolXYZS : 
				count += sprintf ((bufferIndex + count), "%s x,%s y,%s z,%s s", 
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name );	
				break;
			case kNPubyteRGBA : 
				count += sprintf ((bufferIndex + count), "%s r,%s g,%s b,%s a", 
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name );	
				break;
			case kNPubyteRGB : 
				count += sprintf ((bufferIndex + count), "%s r,%s g,%s b", 
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name );	
				break;
			case kNPfloatRGBA : 
				count += sprintf ((bufferIndex + count), "%s r,%s g,%s b,%s a", 
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name );	
				break;
			case kNPfloatRGB : 
				count += sprintf ((bufferIndex + count), "%s r,%s g,%s b", 
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name,
									data->map.typeMapPin[i].name );	
				break;

			default :				// same as case 0
				count += sprintf ((bufferIndex + count), "%s", 
									data->map.typeMapPin[i].name );
				break;
		}
		
		break;

		default : break;
		}


	}

	count += sprintf ((bufferIndex + count), "\n");

	return count;
}

//-----------------------------------------------------------------------------
int npGetMapFile (char* buffer, int count, void* nodeRef, void* dataRef)
{
	int i = 0;
	int type = 0;
	pData data = (pData) dataRef;
	NPplotPtr node = (NPplotPtr) nodeRef;


	data->io.write[kNPplot] = buffer;
	data->io.write[kNPcamera] = buffer + ((kNPmapFileWriteBufferSize * 8) / 16);
	data->io.write[kNPgrid] = buffer + ((kNPmapFileWriteBufferSize * 9) / 16);
	data->io.write[kNPpin] = buffer + ((kNPmapFileWriteBufferSize * 10) / 16);
	data->io.write[kNPchild] = buffer + ((kNPmapFileWriteBufferSize * 12) / 16);
	data->io.write[kNPchannel] = buffer + ((kNPmapFileWriteBufferSize * 14) / 16);


//	for (i=1; i < 20 ;i++)		
//		count += sprintf ((bufferIndex + count), ",%s", data->map.typeMapPlot[i].name);

	count += npWriteMapHeader ((buffer + count), count, kNPplot, dataRef);
	data->io.write[kNPplot] += count;
	
	count += npWriteMapHeader (data->io.write[kNPcamera], count, kNPcamera, dataRef);
	data->io.write[kNPcamera] += count;

	count += npWriteMapHeader (data->io.write[kNPgrid], count, kNPgrid, dataRef);
	data->io.write[kNPgrid] += count;

	count += npWriteMapHeader (data->io.write[kNPpin], count, kNPpin, dataRef);
	data->io.write[kNPpin] += count;

	count += npWriteMapHeader (data->io.write[kNPchild], count, kNPchild, dataRef);
	data->io.write[kNPchild] += count;

	count += npWriteMapHeader (data->io.write[kNPchannel], count, kNPchannel, dataRef);
	data->io.write[kNPchannel] += count;


	// iterate through data map and write out each node
	for (i=0; i < data->map.plotCount; i++)
	{
		node = data->map.plot[i];

		count += npWriteNode ((buffer + count), node, dataRef);

		if (node->childCount)
			count += npMapTraverseTree ((buffer + count), node, dataRef);
	}

	return count;
}

// iterate through plots update translation rotation and scale
//-----------------------------------------------------------------------------
int npExportMapFile (FILE* file, int type, void* dataRef)
{
	int size = 0;

	return size;
}

//-----------------------------------------------------------------------------
int npSeekToNextLine( char* buffer )
{
	char ch = 0;
	int curs = 0;
	while( ch != '\n' )		//Carraige Return at end of Line
	{
		ch = buffer[curs];
		curs++;
	}
	return curs;
}

//-----------------------------------------------------------------------------
//#define FIRSTCHAR 1141
void npLoadMapFile (char* buffer, int wordSize, int size, void* dataRef)
{
	// Mark insert your code here, this is passed in the 

	int i = 0;
	int curs = 0;				//Cursor position in Buffer parsing source
	int scanNumRet = 0;			//sscanf return value. Number of successfuly scanned elements
	int numExpected = 0;		//Expected number for scanNumRet returned from sscanf
	//Group #1
	int id, type, selected,	parentID, branchLevel, childIndex, childCount, 
		channel_x, channel_y, channel_z, channelIndex_x, channelIndex_y, channelIndex_z, 
		averageType_x, averageType_y, averageType_z, sampleInterval_x, sampleInterval_y, sampleInterval_z;
	//Group #2
	float rotate_angle, rotate_x, rotate_y, rotate_z, scale_x, scale_y, scale_z, translate_x, translate_y, translate_z,
		plotOrigin_x, plotOrigin_y, plotOrigin_z, rotateRate_x, rotateRate_y, rotateRate_z, rotateRad_x, rotateRad_y, rotateRad_z;
	//Group #3
	float scaleRate_x, scaleRate_y, scaleRate_z, translateRate_x, translateRate_y, translateRate_z, 
		translateVect_x, translateVect_y, translateVect_z;
	//Group #4
	int surfaceType, shaderType;									
	float lineWidth, pointSize, circle;								
	int colorIndex, color_r, color_g, color_b, color_a, colorFade;
	//Group #5
	int hide, freeze, center, centerPerGrid, autoZoom_x, autoZoom_y, autoZoom_z, scroll, 
		triggerOnHi_x, triggerOnHi_y, triggerOnHi_z, triggerOnLow_x, triggerOnLow_y, triggerOnLow_z;
	//Group #6
	//Group #7


	//
	NPplotPtr node = NULL;
	pData data = (pData) dataRef;

	printf( "%s\n", __FUNCTION__ );
	printf( "size: %d\n", size ); 
	printf( "buffer: %s\n", buffer ); 

//	buffer = &buffer[FIRSTCHAR];		//Seek to First "Real" char. After all the Text Labels. "1" is the first number. It's an ID
	//Seek through first two lines, delimited by carraige returns
	curs = npSeekToNextLine( buffer );	buffer = &buffer[curs];		//First  Line not used
	curs = npSeekToNextLine( buffer );	buffer = &buffer[curs];		//Second Line not used
	printf( "First Real char:%c\n", buffer[0] );

	numExpected = 74;		//21 ints + 19 floats + 9 floats + 11 int+floats + 14 ints
	while( scanNumRet != -1 )
	{
		curs = 0;

		//Scan Whole Line in 
		scanNumRet = sscanf( buffer, 
			"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%d,%d,%f,%f,%f,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d", 
			&id, &type, &id, &selected, &parentID, &branchLevel, &id, &childIndex, &childCount, 
			&channel_x, &channel_y, &channel_z, &channelIndex_x, &channelIndex_y, &channelIndex_z,
			&averageType_x, &averageType_y, &averageType_z, &sampleInterval_x, &sampleInterval_y, &sampleInterval_z,
			&rotate_angle, &rotate_x, &rotate_y, &rotate_z, &scale_x, &scale_y, &scale_z, &translate_x, &translate_y, &translate_z,
			&plotOrigin_x, &plotOrigin_y, &plotOrigin_z, &rotateRate_x, &rotateRate_y, &rotateRate_z, &rotateRad_x, &rotateRad_y, &rotateRad_z,
			&scaleRate_x, &scaleRate_y, &scaleRate_z, &translateRate_x, &translateRate_y, &translateRate_z, 
			&translateVect_x, &translateVect_y, &translateVect_z, 
			&surfaceType, &shaderType, &lineWidth, &pointSize, &circle, &colorIndex, &color_r, &color_g, &color_b, &color_a, &colorFade,
			&hide, &freeze, &center, &centerPerGrid, &autoZoom_x, &autoZoom_y, &autoZoom_z, &scroll, 
			&triggerOnHi_x, &triggerOnHi_y, &triggerOnHi_z, &triggerOnLow_x, &triggerOnLow_y, &triggerOnLow_z
		);
		printf( "scanNumRet: %d\n", scanNumRet );
		printf( "id: %d\n", id );
		if( scanNumRet != numExpected )
		{
			if( scanNumRet == -1 )
				printf( "End of File\n" );
			else
				printf( "WARNING: Bad Format\n" );
		}

		//Find Next Line
		curs = npSeekToNextLine( buffer );
		buffer = &buffer[curs];
	}

/*
	//Group #1 : 21 ints
	n += sprintf ((plotPtr + n), "%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d",
	node->id,					//id is unique to this node
	node->type,		
	node->id,					// node->data->id uses this node id
	
	node->selected,
	parentID,					// parent id replaces pointer to parent
	node->branchLevel,
	node->id,					// node->id is same as childList->id
	node->childIndex,
	node->childCount,
	
	node->channel.x,
	node->channel.y,	
	node->channel.z,
	
	node->channelIndex.x,
	node->channelIndex.y,
	node->channelIndex.z,

	node->averageType.x,
	node->averageType.y,
	node->averageType.z,
	
	node->sampleInterval.x,
	node->sampleInterval.y,
	node->sampleInterval.z );

	//Group #2 : 19 floats
	n += sprintf ((plotPtr + n), ",%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f", 
	node->rotate.angle, 
	node->rotate.x,
	node->rotate.y,
	node->rotate.z,
	
	node->scale.x,
	node->scale.y,
	node->scale.z,
	
	node->translate.x,
	node->translate.y,
	node->translate.z,

	node->plotOrigin.x,
	node->plotOrigin.y,
	node->plotOrigin.z,
	
	node->rotateRate.x,
	node->rotateRate.y,
	node->rotateRate.z,

	node->rotateRad.x,
	node->rotateRad.y,
	node->rotateRad.z );

	//Group #3 : 9 floats
	n += sprintf ((plotPtr + n), ",%f,%f,%f,%f,%f,%f,%f,%f,%f", 
	node->scaleRate.x,
	node->scaleRate.y,
	node->scaleRate.z,

	node->translateRate.x,
	node->translateRate.y,
	node->translateRate.z,

	node->translateVect.x,
	node->translateVect.y,
	node->translateVect.z
	 );

	//Group #4 : 11 ints+floats
	n += sprintf ((plotPtr + n), ",%d,%d,%f,%f,%f,%d,%d,%d,%d,%d,%d",
	node->surfaceType,
	node->shaderType,

	node->lineWidth,
	node->pointSize,
	node->circle,

	node->colorIndex,
	
	node->color.r,
	node->color.g,
	node->color.b,
	node->color.a,

	node->colorFade );

	//Group #5 : 14 ints
	n+= sprintf ((plotPtr + n), ",%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d",
	node->hide,
	node->freeze,
	node->center,
	node->centerPerGrid,

	node->autoZoom.x,
	node->autoZoom.y,
	node->autoZoom.z,

	node->scroll,
	
	node->triggerOnHi.x,
	node->triggerOnHi.y,
	node->triggerOnHi.z,

	node->triggerOnLow.x,
	node->triggerOnLow.y,
	node->triggerOnLow.z );

	//Group #6 : 13 floats + 1 int
	n += sprintf ((plotPtr + n), ",%f,%f,%f,%f,%f,%f,%d,%f,%f,%f,%f,%f,%f",
	node->triggerLevelHi.x,
	node->triggerLevelHi.y,
	node->triggerLevelHi.z,

	node->triggerLevelLow.x,
	node->triggerLevelLow.y,
	node->triggerLevelLow.z,

	node->triggerCommand,
	
	node->proximity.x,
	node->proximity.y,
	node->proximity.z,
	
	node->proximityRadius.x,
	node->proximityRadius.y,
	node->proximityRadius.z );

	//Group #7 : 7 ints
	n += sprintf ((plotPtr + n), ",%d,%d,%d,%d,%d,%d,%d\n",
	node->drawCurrentValue.x,
	node->drawCurrentValue.y,
	node->drawCurrentValue.z,
	node->selectionSet.x,
	node->selectionSet.y,
	node->selectionSet.z,

	node->size
	);	

	*/

}


//-----------------------------------------------------------------------------
void npSetMapFile (char* fileName, int wordSize, int size, void* dataRef)
{
	FILE* file = NULL;
	size_t count = 0;
	int i = 0;
	int byteCount = 0;
	char* bufferIndex = 0;

	
	NPplotPtr node = NULL;
	pData data = (pData) dataRef;

	char* str = (char*)malloc(260);
	char* path = (char*)malloc(260);
	char* buffer = (char*)malloc(kNPmapFileWriteBufferSize);

	if (buffer == NULL)
	{
		printf ("4214 error malloc failed cannot write file");
		npFileClose(file, dataRef);
		free (str);
		free (path);
		free (buffer);
		return;
	}

	strcpy (str, fileName);

	// get the Current Working Directory
	npGetCWD (path, &size);

	printf("CWD path : ");
	for(i = 0; i < size; i++)
		printf ("%c", path[i]);
	printf("\n");

	printf("\nRequired Length : %d\n", (strlen(path) + strlen(fileName))); 
	str = strcat(path, fileName);


	// open/create and write to the file, "w+" overwrites existing files
	file = npFileOpen(str, "r", dataRef);

	if( file != NULL )   // check to see if file exists		 debug, ll
	{
		printf("\nReading File\n"); // was "File is not NULL"			debug, zz
		npFileRewind(file);
		count = npFileRead(buffer, 1, kNPmapFileWriteBufferSize, file, dataRef);
		printf("\nSuccessfully Read %d bytes \n", count);

		//print the File Contents
		printf("File Contents : \n");
		size = count;
		if ( size > 2048) size = 2048;
		for( i = 0; i < size; i++ )
			printf("%c", buffer[i]);

		if (size < 1)
			printf("\n Error : Loading State : File size is zero \n");
		else
		{
			printf("\n Loading State \n");
			npLoadMapFile (buffer, 1, count, dataRef);
		}
	}
	else
		printf("\n File cannot open file \n");


	npFileClose(file, dataRef);
	free (str);
	free (path);
	free (buffer);
	return;
}


//-----------------------------------------------------------------------------
void npTestMapFile (char* fileName, int wordSize, int size, void* dataRef)
{
	FILE* file = NULL;
	size_t count = 0;
	int i = 0;
	int byteCount = 0;
	char* bufferIndex = NULL;
	char* charPtrA = NULL;
	char* charPtrB = NULL;

	NPplotPtr node = NULL;
	pData data = (pData) dataRef;

	char* str = (char*)malloc(260);
	char* buffer = (char*)malloc(kNPmapFileWriteBufferSize);

	if (buffer == NULL || str == NULL)
	{
		printf ("4214 error malloc failed cannot write file");
		free (str);
		return;
	}

	strcpy (str, fileName);  // this is for combining a file path with filename, zz

	// open or create and write to the file, "w" overwrites existing files
	file = npFileOpen(str, "w+", dataRef);

	if( file != NULL )						//check to see if file exists
		printf("\nFile Opened\n");
	else
	{
		printf("\nFile cannot be created\n");							//debug, add err reporting, zz
		return;
	}
	
	bufferIndex = buffer + strlen(buffer);
	bufferIndex += sprintf(bufferIndex, "\n");		//begin file with new line

	// fill the buffer with the map of current state
	count = npGetMapFile (bufferIndex, count, node, dataRef);

	// write the nodes to file
	printf("\nSaving State\n");
	count += npFileWrite (buffer, 1, strlen(buffer), file, dataRef);		//debug, strlen should be count instead, zz

	// write pin data
	charPtrA = ( buffer + ((kNPmapFileWriteBufferSize * 10) / 16));		//debug, bizarre way of merging multiple arrays together, zz
	byteCount = (int)(data->io.write[kNPpin] - charPtrA);
	count += npFileWrite (charPtrA, 1, byteCount, file, dataRef);

	// write child id lists
	charPtrA = ( buffer + ((kNPmapFileWriteBufferSize * 12) / 16));
	byteCount = (int)(data->io.write[kNPchild] - charPtrA);
	count += npFileWrite (charPtrA, 1, byteCount, file, dataRef);

	// write camera data
/*	charPtrA = ( buffer + ((kNPmapFileWriteBufferSize * 8) / 16));
	byteCount = (int)(data->io.write[kNPcamera] - charPtrA);
	count += npFileWrite (charPtrA, 1, 100, file, dataRef);
*/

	// print out the byte count and first few bytes of the file
	if (count)	//== strlen(buffer) )
	{
		printf("\nSuccessfully Wrote %d bytes to file\n", count);
		
		// read the file
		npFileRewind(file);
		size = npFileRead (buffer, 1, 256, file, dataRef);

		//Print File Contents
		printf("File Contents : \n");
		for( i = 0; i < size; i++ )
			printf("%c", buffer[i]);
	}
	else
	{
		printf("\nFile Write Failure\n");
		npFileClose(file, dataRef);
		return; // Not working, return -1
	}


	npFileClose(file, dataRef);
	free (str);
	free (buffer);
	return;
}


//-----------------------------------------------------------------------------
// if not saved then opens SaveAs dialog before reseting app
void npCloseFileMapFile (int size, void* dataRef)
{

}


//-----------------------------------------------------------------------------
int npGetMapFileAttrib (int attrib, void* dataRef)
{
	switch (attrib)
	{
	case kNPversion: ; break;
	case kNPtableCount: ; break;	// number of struct equivalent data tables 
	case kNPrecordCount: ; break;	// total number of records
	case kNPnodeCount: ; break;
	case kNPgraphCount: ; break;	// number of graph elements connect nodes
	case kNPsize: ; break;			// RAM used by map file

	default : break;
	}
}

//-----------------------------------------------------------------------------
void npSetMapFileAttrib (int attrib, void* dataRef)
{


}




//-----------------------------------------------------------------------------
// keep track of a bufferIndex for each list
// write list of children
// write node or plot list
// write node specific data to a separate list, ie pin, camera, etc
// write list of channels,						channels NOT YET IMPLEMENTED, debug, zz
//-----------------------------------------------------------------------------
int npWriteNode (char* buffer, void* nodeRef, void* dataRef)
{
	int i			= 0;
	int n			= 0;
	int total		= 0;
	int parentID	= 0;

	pData data = (pData) dataRef;

	// pointers to the node data, update to be a generic list, debug, zz
	char* plotPtr		= data->io.write[kNPplot];		//	same as buffer
	char* cameraPtr		= data->io.write[kNPcamera];	//	kNPmapFileWriteBufferSize * 8 / 16;
	char* gridPtr		= data->io.write[kNPgrid];		//	kNPmapFileWriteBufferSize * 9 / 16;
	char* pinPtr		= data->io.write[kNPpin];		//	kNPmapFileWriteBufferSize * 10 / 16;
	char* childPtr		= data->io.write[kNPchild];		//	kNPmapFileWriteBufferSize * 12 / 16;
	char* channelPtr	= data->io.write[kNPchannel];	//	kNPmapFileWriteBufferSize * 14 / 16;


	NPplotPtr node = (NPplotPtr) nodeRef;
	NPplotPtr parent = NULL;
	NPplotPtr child = NULL;

	NPcameraPtr camera = NULL;
	NPgridPtr grid = NULL;
	NPpinPtr pin = NULL;

	parent = node;
	if(node->parent != NULL)
	{
		parent = node->parent;
		parentID = parent->id;
	}

	if (0)//buffer != plotPtr)												// debug, zz
		printf ("\n writePtr %p %p %p %p %p %p \n", 
				plotPtr, cameraPtr, gridPtr, pinPtr, childPtr, channelPtr);


	// create a list of child->id, the list id is the same as this node->id
	// used to store memory pointers as an id list for file or db exchange
	if (node->childCount)
	{
		n += sprintf ((childPtr + n), "%d,", node->id );

		for (i=0; i < node->childCount; i++)
		{
			child = node->child[i];

			if (child != NULL)
			{
				// if first item, no comma in front, otherwise insert a comma
				if (i == 0)
					n += sprintf ((childPtr + n), "%d", child->id );
				else
					n += sprintf ((childPtr + n), ",%d", child->id );
			}
		}
	
		n += sprintf ((childPtr + n), "\n", node->id );

		data->io.write[kNPchild] += n;						//					debug zz
		total += n;
		n = 0;
	}



	n += sprintf ((plotPtr + n), "%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d",
	node->id,					//id is unique to this node
	node->type,		
	node->id,					// node->data->id uses this node id
	
	node->selected,
	parentID,					// parent id replaces pointer to parent
	node->branchLevel,
	node->id,					// node->id is same as childList->id
	node->childIndex,
	node->childCount,
	
	node->channel.x,
	node->channel.y,	
	node->channel.z,
	
	node->channelIndex.x,
	node->channelIndex.y,
	node->channelIndex.z,

	node->averageType.x,
	node->averageType.y,
	node->averageType.z,
	
	node->sampleInterval.x,
	node->sampleInterval.y,
	node->sampleInterval.z );


	n += sprintf ((plotPtr + n), ",%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f", 
	node->rotate.angle, 
	node->rotate.x,
	node->rotate.y,
	node->rotate.z,
	
	node->scale.x,
	node->scale.y,
	node->scale.z,
	
	node->translate.x,
	node->translate.y,
	node->translate.z,

	node->plotOrigin.x,
	node->plotOrigin.y,
	node->plotOrigin.z,
	
	node->rotateRate.x,
	node->rotateRate.y,
	node->rotateRate.z,

	node->rotateRad.x,
	node->rotateRad.y,
	node->rotateRad.z );

	n += sprintf ((plotPtr + n), ",%f,%f,%f,%f,%f,%f,%f,%f,%f", 
	node->scaleRate.x,
	node->scaleRate.y,
	node->scaleRate.z,

	node->translateRate.x,
	node->translateRate.y,
	node->translateRate.z,

	node->translateVect.x,
	node->translateVect.y,
	node->translateVect.z
	 );

	n += sprintf ((plotPtr + n), ",%d,%d,%f,%f,%f,%d,%d,%d,%d,%d,%d",
	node->surfaceType,
	node->shaderType,

	node->lineWidth,
	node->pointSize,
	node->circle,

	node->colorIndex,
	
	node->color.r,
	node->color.g,
	node->color.b,
	node->color.a,

	node->colorFade );

	n+= sprintf ((plotPtr + n), ",%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d",
	node->hide,
	node->freeze,
	node->center,
	node->centerPerGrid,

	node->autoZoom.x,
	node->autoZoom.y,
	node->autoZoom.z,

	node->scroll,
	
	node->triggerOnHi.x,
	node->triggerOnHi.y,
	node->triggerOnHi.z,

	node->triggerOnLow.x,
	node->triggerOnLow.y,
	node->triggerOnLow.z );

	n += sprintf ((plotPtr + n), ",%f,%f,%f,%f,%f,%f,%d,%f,%f,%f,%f,%f,%f",
	node->triggerLevelHi.x,
	node->triggerLevelHi.y,
	node->triggerLevelHi.z,

	node->triggerLevelLow.x,
	node->triggerLevelLow.y,
	node->triggerLevelLow.z,

	node->triggerCommand,
	
	node->proximity.x,
	node->proximity.y,
	node->proximity.z,
	
	node->proximityRadius.x,
	node->proximityRadius.y,
	node->proximityRadius.z );

	n += sprintf ((plotPtr + n), ",%d,%d,%d,%d,%d,%d,%d\n",
	node->drawCurrentValue.x,
	node->drawCurrentValue.y,
	node->drawCurrentValue.z,
	node->selectionSet.x,
	node->selectionSet.y,
	node->selectionSet.z,

	node->size
	);	

	data->io.write[kNPplot] += n;						//					debug zz
	total += n;
	n = 0;

	if (node->type == kPlotPin)
	{
		pin = (NPpinPtr) node->data;

		n += sprintf ((pinPtr + n), "%d,%d,%d,%f,%f,%d,%d,%d,%d,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%d\n",
			pin->id,					//id is unique to this node
			pin->type,		
			pin->textureID,					// node->data->id uses this node id
			pin->innerRadius,
			pin->outerRadius,
			pin->radiusRatioIndex,
			pin->screenSizeIndex,
			pin->slices,
			pin->stacks,
			pin->translateTex.x,
			pin->translateTex.y,
			pin->translateTex.z,
			pin->scaleTex.x,
			pin->scaleTex.y,
			pin->scaleTex.z,
			pin->rotateTex.x,
			pin->rotateTex.y,
			pin->rotateTex.z,
			pin->rotateTex.angle,
			pin->size );
	}

	data->io.write[kNPpin] += n;						//					debug zz
	total += n;
	n = 0;

	if (node->type == kPlotCamera)
	{
		camera = (NPcameraPtr) node->data;

		n += sprintf ((pinPtr + n), "%d,%d,%d,%d,%d,%d,%f,%f,%d,%d,%d,%f,%f,%f,%f,%f,%d,%d\n",
			camera->id,					//id is unique to this node
			camera->type,

			camera->textureID,					// node->data->id uses this node id

			camera->format,
			camera->interlaced,
			camera->stereo,
			camera->aspectRatio,
			camera->fps,
			camera->colorSpace,
			camera->width,
			camera->height,

			camera->fov,
			camera->clipNear,
			camera->clipFar,
			camera->aperture,
			camera->exposure,
			camera->sensorType,

			camera->size );
	}

	data->io.write[kNPcamera] += n;						//					debug zz
	total += n;
	n = 0;

	return n;
}

void npSetChannelFile (char* fileName, int wordSize, int size, void* dataRef)
{}