//------------------------------------------------------------------------------
//
//  npgl.c
//  ANTZ application based on NPE
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------


#include "npgl.h"

#include "../SOIL.h"

#include "../npctrl.h"


GLuint texture;														// texture, debug zz

int previousX;
int previousY;

void npGLResizeScene (int width, int height);

void npCopyGPUtoCPU (void* dataRef);



//------------------------------------------------------------------------------
void npInitGL(void* dataRef) 
{
	previousX = 0;
	previousY = 0;


	npInitGLDraw (dataRef);

//	glShadeModel (GL_SMOOTH);
//	glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
//	glClearDepth (1.0f);

	glEnable (GL_DEPTH_TEST);

//	glDepthFunc (GL_LEQUAL);
//	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	  // load our texture												// texture, debug zz
//  texture = LoadTextureRAW( "texture.raw", TRUE );
  texture = SOIL_load_OGL_texture
	(
		"map00001.jpg",
		SOIL_LOAD_AUTO,
		SOIL_CREATE_NEW_ID,
		SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT
	);																// texture, debug zz

}


// currently just single window support, add multi-screen capability later... debug zz
// redbook has info on how to calculate the projections
//------------------------------------------------------------------------------
void npGLResizeScene (int width, int height)
{
	int camIndex = 1;	//update with list in data->io->view.camIndex[]		debug zz
	GLfloat ratio = 1.0;

	pData data = npDataGetDataRef();

	NPplotPtr camPlot = data->map.plot[camIndex];
	NPcameraPtr camData = camPlot->data;

	if (height > 0)							// Prevent a divide by zero
		camData->aspectRatio = (GLfloat)width / (GLfloat)height;

	// Reset the coordinate system before modifying
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	
	// Set the viewport to be the entire window
    glViewport(0, 0, width, height);

	// Set the clipping volume
	gluPerspective (camData->fov, camData->aspectRatio, 
					camData->clipNear, camData->clipFar);
	glMatrixMode (GL_MODELVIEW);

	npGLDrawScene (data);
}


//------------------------------------------------------------------------------
void npGLDrawScene (void* dataRef)
{
	int	camIndex = 1;
	GLfloat upX = 0.0f, upY = 0.0f, upZ = 1.0f;
	
	GLfloat ambient[] = { 0.3f, 0.3f, 0.3f, 1.0f };
	GLfloat diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat specular[] = { 0.3f, 0.3f, 0.3f, 1.0f };
	GLfloat position[] = { -150.0f, 150.0f, -150.0f, 1.0f };
	GLfloat positionB[] = { 150.0f, -150.0f, 150.0f, 1.0f };
	GLfloat positionC[] = { 0.0f, 0.0f, 15.0f, 1.0f };

	GLfloat globalAmbient[] = { 0.3f, 0.3f, 0.3f, 1.0f };

	GLfloat materialSpecular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat materialShininess[] = { 128.0f };
	GLfloat materialAmbient[] = { 0.25f, 0.25f, 0.25f, 1.0f };
	GLfloat materialDiffuse[] = { 0.4f, 0.4f, 0.4f, 1.0f };
	GLfloat localAmbient[] = { 0.0f, 0.0f, 0.0f, 1.0f };


GLfloat angle = 0.0;
GLfloat redDiffuseMaterial[] = {1.0f, 1.0f, 1.0f}; //set the material to red
GLfloat whiteSpecularMaterial[] = {1.0f, 1.0f, 1.0f}; //set the material to white
GLfloat greenEmissiveMaterial[] = {0.1f, 0.1f, 0.1f}; //set the material to green
GLfloat whiteSpecularLight[] = {1.0f, 1.0f, 1.0f}; //set the light specular to white
GLfloat blackAmbientLight[] = {0.0f, 0.0f, 0.0f}; //set the light ambient to black
GLfloat whiteDiffuseLight[] = {1.0f, 1.0f, 1.0f}; //set the diffuse light to white
GLfloat blankMaterial[] = {0.0f, 0.0f, 0.0f}; //set the diffuse light to white
GLfloat mShininess[] = {128}; //set the shininess of the material

GLfloat redNotGreenNotBlue[] = {1.0f, -1.0f, -1.0f}; //set the material to red

	pData data = (pData) dataRef;
	NPplotPtr camPlot = data->map.plot[camIndex];
	NPcameraPtr camData = camPlot->data;

	NPplotPtr gridPlot = data->map.plot[2];					// texture, debug zz
	NPgridPtr gridData = gridPlot->data;

	GLfloat gridX = 0.0f, gridY = 0.0f;

	//lighting
//	glLightModelfv (GL_LIGHT_MODEL_AMBIENT, globalAmbient);
	
	npCtrlUpdate (dataRef);	  //update data, positions, physics, interactions...

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);

	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);
	glEnable (GL_LIGHT1);
	glEnable (GL_LIGHT2);

	glLightfv (GL_LIGHT0, GL_AMBIENT, ambient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, diffuse);
	glLightfv (GL_LIGHT0, GL_SPECULAR, specular);
	glLightfv (GL_LIGHT0, GL_POSITION, position);
 
	glLightfv(GL_LIGHT0, GL_SPECULAR, whiteSpecularLight);
    glLightfv(GL_LIGHT0, GL_AMBIENT, blackAmbientLight);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, whiteDiffuseLight);

	glLightfv (GL_LIGHT1, GL_AMBIENT, ambient);
	glLightfv (GL_LIGHT1, GL_DIFFUSE, diffuse);
	glLightfv (GL_LIGHT1, GL_SPECULAR, specular);
	glLightfv (GL_LIGHT1, GL_POSITION, positionB);
 
	glLightfv(GL_LIGHT1, GL_SPECULAR, whiteSpecularLight);
    glLightfv(GL_LIGHT1, GL_AMBIENT, blackAmbientLight);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, whiteDiffuseLight);

	glLightfv (GL_LIGHT2, GL_AMBIENT, blackAmbientLight);
	glLightfv (GL_LIGHT2, GL_DIFFUSE, redNotGreenNotBlue);
	glLightfv (GL_LIGHT2, GL_SPECULAR, redNotGreenNotBlue);
	glLightfv (GL_LIGHT2, GL_POSITION, positionC);
 
	glLightfv(GL_LIGHT2, GL_SPECULAR, redNotGreenNotBlue);
    glLightfv(GL_LIGHT2, GL_AMBIENT, blackAmbientLight);
    glLightfv(GL_LIGHT2, GL_DIFFUSE, redNotGreenNotBlue);
/*
	glMaterialfv ( GL_FRONT, GL_SPECULAR, materialSpecular);
	glMaterialfv ( GL_FRONT, GL_SHININESS, materialShininess);
	glMaterialfv ( GL_FRONT, GL_DIFFUSE, materialDiffuse);
	glMaterialfv ( GL_FRONT, GL_AMBIENT, materialAmbient);
*/
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, whiteSpecularMaterial);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mShininess);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, redDiffuseMaterial);
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, greenEmissiveMaterial);

	glEnable (GL_COLOR_MATERIAL);

//	glBlendFunc(GL_SRC_ALPHA, GL_DST_COLOR);//GL_ONE_MINUS_SRC_ALPHA);
//	glEnable(GL_BLEND);													// debug, zz
//	glAlphaFunc(GL_GREATER,0.1f);
//	glEnable(GL_ALPHA_TEST);

	//	glDisable (GL_DEPTH_TEST);


	glShadeModel (GL_SMOOTH);

	npDrawPlots (dataRef);				//draws mesh, graphs, pins, etc...

      // setup texture mapping									// texture, debug zz

	//gridPlot = data->map.plot[2];

	gridX = gridPlot->scale.x * 5.0f * (GLfloat) gridData->segments.x;
	gridY = gridPlot->scale.y * 5.0f * (GLfloat) gridData->segments.y;

	glDisable (GL_LIGHTING);
      glEnable( GL_TEXTURE_2D );
      glBindTexture( GL_TEXTURE_2D, texture );

	  glColor4f (1.0f, 1.0f, 1.0f, 1.0f);
      glPushMatrix();
      glRotatef( 0.0f, 0.0f, 0.0f, 1.0f );
      glBegin( GL_QUADS );
      glTexCoord2d(0.0,0.0); glVertex3d(-gridX,-gridY,-0.1f);
      glTexCoord2d(1.0,0.0); glVertex3d(+gridX,-gridY,-0.1f);
      glTexCoord2d(1.0,1.0); glVertex3d(+gridX,+gridY,-0.1f);
      glTexCoord2d(0.0,1.0); glVertex3d(-gridX,+gridY,-0.1f);
      glEnd();
      glPopMatrix();
	   glDisable( GL_TEXTURE_2D );
	glEnable (GL_LIGHTING);										//end texture, debug zz

	glLoadIdentity();
	gluLookAt (	camPlot->translate.x, 
				camPlot->translate.y,
				camPlot->translate.z, 
				camPlot->translate.x + camPlot->rotate.x,
				camPlot->translate.y + camPlot->rotate.y,
				camPlot->translate.z + camPlot->rotate.z,
				upX, upY, upZ );

	npDrawHUD (dataRef);				// draws overlays, grids, text...

	npCopyGPUtoCPU (dataRef);			// copies display to main CPU memory
}


// copy the rasterized display to the main system memory (CPU RAM or Drive)
//------------------------------------------------------------------------------
void npCopyGPUtoCPU (void* dataRef)
{

}


//------------------------------------------------------------------------------
void npMouseEvent (int button, int state, int x, int y)
{
	pData data = (pData) npDataGetDataRef();

	previousX = x;
	previousY = y;

	if (button == GLUT_LEFT_BUTTON && state == GLUT_UP)
	{
		data->io.mouseDeltaX = 0.0f;
		data->io.mouseDeltaY = 0.0f;
	}
}

//------------------------------------------------------------------------------
void npMouseMotion (int x, int y)
{
	pData data = (pData) npDataGetDataRef();


	data->io.mouseDeltaX = (GLfloat)(x - previousX);
	data->io.mouseDeltaY = (GLfloat)(y - previousY);

//	printf ("\n %d %d", data->io.mouseDeltaX, data->io.mouseDeltaY);

	previousX = x;
	previousY = y;
}

void npGLSurface (int surfaceType, void* dataRef)
{

}